from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.list_jobs_order_by import ListJobsOrderBy
from ...models.scaleway_qaas_v1_alpha_1_list_jobs_response import (
    ScalewayQaasV1Alpha1ListJobsResponse,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    tags: Union[Unset, list[str]] = UNSET,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListJobsOrderBy] = ListJobsOrderBy.CREATED_AT_DESC,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_tags: Union[Unset, list[str]] = UNSET
    if not isinstance(tags, Unset):
        json_tags = tags

    params["tags"] = json_tags

    params["page"] = page

    params["page_size"] = page_size

    json_order_by: Union[Unset, str] = UNSET
    if not isinstance(order_by, Unset):
        json_order_by = order_by.value

    params["order_by"] = json_order_by

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/qaas/v1alpha1/jobs",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[ScalewayQaasV1Alpha1ListJobsResponse]:
    if response.status_code == 200:
        response_200 = ScalewayQaasV1Alpha1ListJobsResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[ScalewayQaasV1Alpha1ListJobsResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    tags: Union[Unset, list[str]] = UNSET,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListJobsOrderBy] = ListJobsOrderBy.CREATED_AT_DESC,
) -> Response[ScalewayQaasV1Alpha1ListJobsResponse]:
    """List all jobs within a project or session

     Retrieve information about all jobs within a given project or session.

    Args:
        tags (Union[Unset, list[str]]): List jobs with these tags.
        page (Union[Unset, int]): Page number.
        page_size (Union[Unset, int]): Maximum number of jobs to return per page.
        order_by (Union[Unset, ListJobsOrderBy]): Sort order of the returned jobs. Default:
            ListJobsOrderBy.CREATED_AT_DESC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ScalewayQaasV1Alpha1ListJobsResponse]
    """

    kwargs = _get_kwargs(
        tags=tags,
        page=page,
        page_size=page_size,
        order_by=order_by,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    tags: Union[Unset, list[str]] = UNSET,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListJobsOrderBy] = ListJobsOrderBy.CREATED_AT_DESC,
) -> Optional[ScalewayQaasV1Alpha1ListJobsResponse]:
    """List all jobs within a project or session

     Retrieve information about all jobs within a given project or session.

    Args:
        tags (Union[Unset, list[str]]): List jobs with these tags.
        page (Union[Unset, int]): Page number.
        page_size (Union[Unset, int]): Maximum number of jobs to return per page.
        order_by (Union[Unset, ListJobsOrderBy]): Sort order of the returned jobs. Default:
            ListJobsOrderBy.CREATED_AT_DESC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ScalewayQaasV1Alpha1ListJobsResponse
    """

    return sync_detailed(
        client=client,
        tags=tags,
        page=page,
        page_size=page_size,
        order_by=order_by,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    tags: Union[Unset, list[str]] = UNSET,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListJobsOrderBy] = ListJobsOrderBy.CREATED_AT_DESC,
) -> Response[ScalewayQaasV1Alpha1ListJobsResponse]:
    """List all jobs within a project or session

     Retrieve information about all jobs within a given project or session.

    Args:
        tags (Union[Unset, list[str]]): List jobs with these tags.
        page (Union[Unset, int]): Page number.
        page_size (Union[Unset, int]): Maximum number of jobs to return per page.
        order_by (Union[Unset, ListJobsOrderBy]): Sort order of the returned jobs. Default:
            ListJobsOrderBy.CREATED_AT_DESC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ScalewayQaasV1Alpha1ListJobsResponse]
    """

    kwargs = _get_kwargs(
        tags=tags,
        page=page,
        page_size=page_size,
        order_by=order_by,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    tags: Union[Unset, list[str]] = UNSET,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListJobsOrderBy] = ListJobsOrderBy.CREATED_AT_DESC,
) -> Optional[ScalewayQaasV1Alpha1ListJobsResponse]:
    """List all jobs within a project or session

     Retrieve information about all jobs within a given project or session.

    Args:
        tags (Union[Unset, list[str]]): List jobs with these tags.
        page (Union[Unset, int]): Page number.
        page_size (Union[Unset, int]): Maximum number of jobs to return per page.
        order_by (Union[Unset, ListJobsOrderBy]): Sort order of the returned jobs. Default:
            ListJobsOrderBy.CREATED_AT_DESC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ScalewayQaasV1Alpha1ListJobsResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            tags=tags,
            page=page,
            page_size=page_size,
            order_by=order_by,
        )
    ).parsed



import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.scaleway_qaas_v1_alpha_1_process_status import (
    ScalewayQaasV1Alpha1ProcessStatus,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="ScalewayQaasV1Alpha1Process")


@_attrs_define
class ScalewayQaasV1Alpha1Process:
    """
    Attributes:
        id (Union[Unset, str]): Unique ID of the process.
        name (Union[Unset, str]): Name of the process.
        application_id (Union[None, Unset, str]): Application ID for which the process has been created.
        platform_id (Union[None, Unset, str]): Platform ID for which the process has been created.
        attached_session_ids (Union[Unset, list[str]]): List of sessions generated by the process.
        created_at (Union[None, Unset, datetime.datetime]): Tme at which the process was created. (RFC 3339 format)
            Example: 2022-03-22T12:34:56.123456Z.
        started_at (Union[None, Unset, datetime.datetime]): Time at which the process started. (RFC 3339 format)
            Example: 2022-03-22T12:34:56.123456Z.
        updated_at (Union[None, Unset, datetime.datetime]): Time at which the process was updated. (RFC 3339 format)
            Example: 2022-03-22T12:34:56.123456Z.
        finished_at (Union[None, Unset, datetime.datetime]): Time at which the process was finished. (RFC 3339 format)
            Example: 2022-03-22T12:34:56.123456Z.
        status (Union[Unset, ScalewayQaasV1Alpha1ProcessStatus]): Status of the process. Default:
            ScalewayQaasV1Alpha1ProcessStatus.UNKNOWN_STATUS.
        project_id (Union[Unset, str]): Project ID in which the process has been created. (UUID format) Example:
            6170692e-7363-616c-6577-61792e636f6d.
        tags (Union[Unset, list[str]]): Tags of the process.
        progress (Union[None, Unset, int]): Progress of the process, from 0 to 1.
        progress_message (Union[None, Unset, str]): Any progress of the process.
        input_ (Union[None, Unset, str]): Input payload of the process as JSON string.
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    application_id: Union[None, Unset, str] = UNSET
    platform_id: Union[None, Unset, str] = UNSET
    attached_session_ids: Union[Unset, list[str]] = UNSET
    created_at: Union[None, Unset, datetime.datetime] = UNSET
    started_at: Union[None, Unset, datetime.datetime] = UNSET
    updated_at: Union[None, Unset, datetime.datetime] = UNSET
    finished_at: Union[None, Unset, datetime.datetime] = UNSET
    status: Union[Unset, ScalewayQaasV1Alpha1ProcessStatus] = (
        ScalewayQaasV1Alpha1ProcessStatus.UNKNOWN_STATUS
    )
    project_id: Union[Unset, str] = UNSET
    tags: Union[Unset, list[str]] = UNSET
    progress: Union[None, Unset, int] = UNSET
    progress_message: Union[None, Unset, str] = UNSET
    input_: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        application_id: Union[None, Unset, str]
        if isinstance(self.application_id, Unset):
            application_id = UNSET
        else:
            application_id = self.application_id

        platform_id: Union[None, Unset, str]
        if isinstance(self.platform_id, Unset):
            platform_id = UNSET
        else:
            platform_id = self.platform_id

        attached_session_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.attached_session_ids, Unset):
            attached_session_ids = self.attached_session_ids

        created_at: Union[None, Unset, str]
        if isinstance(self.created_at, Unset):
            created_at = UNSET
        elif isinstance(self.created_at, datetime.datetime):
            created_at = self.created_at.isoformat()
        else:
            created_at = self.created_at

        started_at: Union[None, Unset, str]
        if isinstance(self.started_at, Unset):
            started_at = UNSET
        elif isinstance(self.started_at, datetime.datetime):
            started_at = self.started_at.isoformat()
        else:
            started_at = self.started_at

        updated_at: Union[None, Unset, str]
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        finished_at: Union[None, Unset, str]
        if isinstance(self.finished_at, Unset):
            finished_at = UNSET
        elif isinstance(self.finished_at, datetime.datetime):
            finished_at = self.finished_at.isoformat()
        else:
            finished_at = self.finished_at

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        project_id = self.project_id

        tags: Union[Unset, list[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        progress: Union[None, Unset, int]
        if isinstance(self.progress, Unset):
            progress = UNSET
        else:
            progress = self.progress

        progress_message: Union[None, Unset, str]
        if isinstance(self.progress_message, Unset):
            progress_message = UNSET
        else:
            progress_message = self.progress_message

        input_: Union[None, Unset, str]
        if isinstance(self.input_, Unset):
            input_ = UNSET
        else:
            input_ = self.input_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if application_id is not UNSET:
            field_dict["application_id"] = application_id
        if platform_id is not UNSET:
            field_dict["platform_id"] = platform_id
        if attached_session_ids is not UNSET:
            field_dict["attached_session_ids"] = attached_session_ids
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if finished_at is not UNSET:
            field_dict["finished_at"] = finished_at
        if status is not UNSET:
            field_dict["status"] = status
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if tags is not UNSET:
            field_dict["tags"] = tags
        if progress is not UNSET:
            field_dict["progress"] = progress
        if progress_message is not UNSET:
            field_dict["progress_message"] = progress_message
        if input_ is not UNSET:
            field_dict["input"] = input_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_application_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        application_id = _parse_application_id(d.pop("application_id", UNSET))

        def _parse_platform_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        platform_id = _parse_platform_id(d.pop("platform_id", UNSET))

        attached_session_ids = cast(list[str], d.pop("attached_session_ids", UNSET))

        def _parse_created_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                created_at_type_0 = isoparse(data)

                return created_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        created_at = _parse_created_at(d.pop("created_at", UNSET))

        def _parse_started_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                started_at_type_0 = isoparse(data)

                return started_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        started_at = _parse_started_at(d.pop("started_at", UNSET))

        def _parse_updated_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)

                return updated_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        updated_at = _parse_updated_at(d.pop("updated_at", UNSET))

        def _parse_finished_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                finished_at_type_0 = isoparse(data)

                return finished_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        finished_at = _parse_finished_at(d.pop("finished_at", UNSET))

        _status = d.pop("status", UNSET)
        status: Union[Unset, ScalewayQaasV1Alpha1ProcessStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ScalewayQaasV1Alpha1ProcessStatus(_status)

        project_id = d.pop("project_id", UNSET)

        tags = cast(list[str], d.pop("tags", UNSET))

        def _parse_progress(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        progress = _parse_progress(d.pop("progress", UNSET))

        def _parse_progress_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        progress_message = _parse_progress_message(d.pop("progress_message", UNSET))

        def _parse_input_(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        input_ = _parse_input_(d.pop("input", UNSET))

        scaleway_qaas_v1_alpha_1_process = cls(
            id=id,
            name=name,
            application_id=application_id,
            platform_id=platform_id,
            attached_session_ids=attached_session_ids,
            created_at=created_at,
            started_at=started_at,
            updated_at=updated_at,
            finished_at=finished_at,
            status=status,
            project_id=project_id,
            tags=tags,
            progress=progress,
            progress_message=progress_message,
            input_=input_,
        )

        scaleway_qaas_v1_alpha_1_process.additional_properties = d
        return scaleway_qaas_v1_alpha_1_process

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListPlatformsPlatformType(str, Enum):
    QPU = "qpu"
    SIMULATOR = "simulator"
    UNKNOWN_TYPE = "unknown_type"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ScalewayQaasV1Alpha1PlatformAvailability(str, Enum):
    AVAILABLE = "available"
    MAINTENANCE = "maintenance"
    SCARCE = "scarce"
    SHORTAGE = "shortage"
    UNKNOWN_AVAILABILITY = "unknown_availability"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ListJobResultsOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.scaleway_qaas_v1_alpha_1_platform_availability import (
    ScalewayQaasV1Alpha1PlatformAvailability,
)
from ..models.scaleway_qaas_v1_alpha_1_platform_technology import (
    ScalewayQaasV1Alpha1PlatformTechnology,
)
from ..models.scaleway_qaas_v1_alpha_1_platform_type import (
    ScalewayQaasV1Alpha1PlatformType,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.scaleway_qaas_v1_alpha_1_platform_hardware import (
        ScalewayQaasV1Alpha1PlatformHardware,
    )
    from ..models.scaleway_qaas_v1_alpha_1_platform_price_per_circuit import (
        ScalewayQaasV1Alpha1PlatformPricePerCircuit,
    )
    from ..models.scaleway_qaas_v1_alpha_1_platform_price_per_hour import (
        ScalewayQaasV1Alpha1PlatformPricePerHour,
    )
    from ..models.scaleway_qaas_v1_alpha_1_platform_price_per_shot import (
        ScalewayQaasV1Alpha1PlatformPricePerShot,
    )


T = TypeVar("T", bound="ScalewayQaasV1Alpha1Platform")


@_attrs_define
class ScalewayQaasV1Alpha1Platform:
    """
    Attributes:
        id (Union[Unset, str]): Unique ID of the platform.
        version (Union[Unset, str]): Version of the platform.
        name (Union[Unset, str]): Name of the platform.
        provider_name (Union[Unset, str]): Provider name of the platform.
        backend_name (Union[Unset, str]): Name of the running backend over the platform (ascella, qsim, aer...).
        type_ (Union[Unset, ScalewayQaasV1Alpha1PlatformType]): Type of the platform. Default:
            ScalewayQaasV1Alpha1PlatformType.UNKNOWN_TYPE.
        technology (Union[Unset, ScalewayQaasV1Alpha1PlatformTechnology]): Technology used by the platform. Default:
            ScalewayQaasV1Alpha1PlatformTechnology.UNKNOWN_TECHNOLOGY.
        max_qubit_count (Union[Unset, int]): Estimated maximum number of qubits supported by the platform.
        max_shot_count (Union[Unset, int]): Maximum number of shots during a circuit execution.
        max_circuit_count (Union[Unset, int]): Maximum number of circuit that can be executed in one call.
        availability (Union[Unset, ScalewayQaasV1Alpha1PlatformAvailability]): Availability of the platform. Default:
            ScalewayQaasV1Alpha1PlatformAvailability.UNKNOWN_AVAILABILITY.
        metadata (Union[Unset, str]): Metadata provided by the platform.
        price_per_hour (Union[Unset, ScalewayQaasV1Alpha1PlatformPricePerHour]): Price to be paid per hour (excluding
            free tiers).
        price_per_shot (Union[Unset, ScalewayQaasV1Alpha1PlatformPricePerShot]): Price to be paid per shot (excluding
            free tiers).
        price_per_circuit (Union[Unset, ScalewayQaasV1Alpha1PlatformPricePerCircuit]): Price to be paid per circuit
            setup before its execution (excluding free tiers).
        hardware (Union[Unset, ScalewayQaasV1Alpha1PlatformHardware]): Specifications of the underlying hardware.
    """

    id: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    provider_name: Union[Unset, str] = UNSET
    backend_name: Union[Unset, str] = UNSET
    type_: Union[Unset, ScalewayQaasV1Alpha1PlatformType] = (
        ScalewayQaasV1Alpha1PlatformType.UNKNOWN_TYPE
    )
    technology: Union[Unset, ScalewayQaasV1Alpha1PlatformTechnology] = (
        ScalewayQaasV1Alpha1PlatformTechnology.UNKNOWN_TECHNOLOGY
    )
    max_qubit_count: Union[Unset, int] = UNSET
    max_shot_count: Union[Unset, int] = UNSET
    max_circuit_count: Union[Unset, int] = UNSET
    availability: Union[Unset, ScalewayQaasV1Alpha1PlatformAvailability] = (
        ScalewayQaasV1Alpha1PlatformAvailability.UNKNOWN_AVAILABILITY
    )
    metadata: Union[Unset, str] = UNSET
    price_per_hour: Union[Unset, "ScalewayQaasV1Alpha1PlatformPricePerHour"] = UNSET
    price_per_shot: Union[Unset, "ScalewayQaasV1Alpha1PlatformPricePerShot"] = UNSET
    price_per_circuit: Union[Unset, "ScalewayQaasV1Alpha1PlatformPricePerCircuit"] = (
        UNSET
    )
    hardware: Union[Unset, "ScalewayQaasV1Alpha1PlatformHardware"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        version = self.version

        name = self.name

        provider_name = self.provider_name

        backend_name = self.backend_name

        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        technology: Union[Unset, str] = UNSET
        if not isinstance(self.technology, Unset):
            technology = self.technology.value

        max_qubit_count = self.max_qubit_count

        max_shot_count = self.max_shot_count

        max_circuit_count = self.max_circuit_count

        availability: Union[Unset, str] = UNSET
        if not isinstance(self.availability, Unset):
            availability = self.availability.value

        metadata = self.metadata

        price_per_hour: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.price_per_hour, Unset):
            price_per_hour = self.price_per_hour.to_dict()

        price_per_shot: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.price_per_shot, Unset):
            price_per_shot = self.price_per_shot.to_dict()

        price_per_circuit: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.price_per_circuit, Unset):
            price_per_circuit = self.price_per_circuit.to_dict()

        hardware: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.hardware, Unset):
            hardware = self.hardware.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if version is not UNSET:
            field_dict["version"] = version
        if name is not UNSET:
            field_dict["name"] = name
        if provider_name is not UNSET:
            field_dict["provider_name"] = provider_name
        if backend_name is not UNSET:
            field_dict["backend_name"] = backend_name
        if type_ is not UNSET:
            field_dict["type"] = type_
        if technology is not UNSET:
            field_dict["technology"] = technology
        if max_qubit_count is not UNSET:
            field_dict["max_qubit_count"] = max_qubit_count
        if max_shot_count is not UNSET:
            field_dict["max_shot_count"] = max_shot_count
        if max_circuit_count is not UNSET:
            field_dict["max_circuit_count"] = max_circuit_count
        if availability is not UNSET:
            field_dict["availability"] = availability
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if price_per_hour is not UNSET:
            field_dict["price_per_hour"] = price_per_hour
        if price_per_shot is not UNSET:
            field_dict["price_per_shot"] = price_per_shot
        if price_per_circuit is not UNSET:
            field_dict["price_per_circuit"] = price_per_circuit
        if hardware is not UNSET:
            field_dict["hardware"] = hardware

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.scaleway_qaas_v1_alpha_1_platform_hardware import (
            ScalewayQaasV1Alpha1PlatformHardware,
        )
        from ..models.scaleway_qaas_v1_alpha_1_platform_price_per_circuit import (
            ScalewayQaasV1Alpha1PlatformPricePerCircuit,
        )
        from ..models.scaleway_qaas_v1_alpha_1_platform_price_per_hour import (
            ScalewayQaasV1Alpha1PlatformPricePerHour,
        )
        from ..models.scaleway_qaas_v1_alpha_1_platform_price_per_shot import (
            ScalewayQaasV1Alpha1PlatformPricePerShot,
        )

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        version = d.pop("version", UNSET)

        name = d.pop("name", UNSET)

        provider_name = d.pop("provider_name", UNSET)

        backend_name = d.pop("backend_name", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, ScalewayQaasV1Alpha1PlatformType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = ScalewayQaasV1Alpha1PlatformType(_type_)

        _technology = d.pop("technology", UNSET)
        technology: Union[Unset, ScalewayQaasV1Alpha1PlatformTechnology]
        if isinstance(_technology, Unset):
            technology = UNSET
        else:
            technology = ScalewayQaasV1Alpha1PlatformTechnology(_technology)

        max_qubit_count = d.pop("max_qubit_count", UNSET)

        max_shot_count = d.pop("max_shot_count", UNSET)

        max_circuit_count = d.pop("max_circuit_count", UNSET)

        _availability = d.pop("availability", UNSET)
        availability: Union[Unset, ScalewayQaasV1Alpha1PlatformAvailability]
        if isinstance(_availability, Unset):
            availability = UNSET
        else:
            availability = ScalewayQaasV1Alpha1PlatformAvailability(_availability)

        metadata = d.pop("metadata", UNSET)

        _price_per_hour = d.pop("price_per_hour", UNSET)
        price_per_hour: Union[Unset, ScalewayQaasV1Alpha1PlatformPricePerHour]
        if isinstance(_price_per_hour, Unset):
            price_per_hour = UNSET
        else:
            price_per_hour = ScalewayQaasV1Alpha1PlatformPricePerHour.from_dict(
                _price_per_hour
            )

        _price_per_shot = d.pop("price_per_shot", UNSET)
        price_per_shot: Union[Unset, ScalewayQaasV1Alpha1PlatformPricePerShot]
        if isinstance(_price_per_shot, Unset):
            price_per_shot = UNSET
        else:
            price_per_shot = ScalewayQaasV1Alpha1PlatformPricePerShot.from_dict(
                _price_per_shot
            )

        _price_per_circuit = d.pop("price_per_circuit", UNSET)
        price_per_circuit: Union[Unset, ScalewayQaasV1Alpha1PlatformPricePerCircuit]
        if isinstance(_price_per_circuit, Unset):
            price_per_circuit = UNSET
        else:
            price_per_circuit = ScalewayQaasV1Alpha1PlatformPricePerCircuit.from_dict(
                _price_per_circuit
            )

        _hardware = d.pop("hardware", UNSET)
        hardware: Union[Unset, ScalewayQaasV1Alpha1PlatformHardware]
        if isinstance(_hardware, Unset):
            hardware = UNSET
        else:
            hardware = ScalewayQaasV1Alpha1PlatformHardware.from_dict(_hardware)

        scaleway_qaas_v1_alpha_1_platform = cls(
            id=id,
            version=version,
            name=name,
            provider_name=provider_name,
            backend_name=backend_name,
            type_=type_,
            technology=technology,
            max_qubit_count=max_qubit_count,
            max_shot_count=max_shot_count,
            max_circuit_count=max_circuit_count,
            availability=availability,
            metadata=metadata,
            price_per_hour=price_per_hour,
            price_per_shot=price_per_shot,
            price_per_circuit=price_per_circuit,
            hardware=hardware,
        )

        scaleway_qaas_v1_alpha_1_platform.additional_properties = d
        return scaleway_qaas_v1_alpha_1_platform

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ScalewayQaasV1Alpha1JobStatus(str, Enum):
    CANCELLED = "cancelled"
    CANCELLING = "cancelling"
    COMPLETED = "completed"
    ERROR = "error"
    RUNNING = "running"
    UNKNOWN_STATUS = "unknown_status"
    WAITING = "waiting"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ScalewayQaasV1Alpha1ProcessStatus(str, Enum):
    CANCELLED = "cancelled"
    CANCELLING = "cancelling"
    COMPLETED = "completed"
    ERROR = "error"
    RUNNING = "running"
    STARTING = "starting"
    UNKNOWN_STATUS = "unknown_status"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ScalewayQaasV1Alpha1SessionOriginType(str, Enum):
    CUSTOMER = "customer"
    PROCESS = "process"
    UNKNOWN_ORIGIN_TYPE = "unknown_origin_type"

    def __str__(self) -> str:
        return str(self.value)



# Copyright 2025 Scaleway
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import randomname

from typing import List, Optional, Dict, Union

from quantum_as_a_service_api_client.models import (
    CreateJobBody,
    CreateJobBodyCircuit,
    CreateSessionBody,
    TerminateSessionBody,
    ScalewayQaasV1Alpha1Platform,
    ScalewayQaasV1Alpha1Job,
    ScalewayQaasV1Alpha1JobResult,
    ScalewayQaasV1Alpha1Session,
)

from quantum_as_a_service_api_client.api.sessions.create_session import (
    sync as _create_session_sync,
)
from quantum_as_a_service_api_client.api.sessions.get_session import (
    sync as _get_session_sync,
)
from quantum_as_a_service_api_client.api.sessions.terminate_session import (
    sync as _terminate_session_sync,
)
from quantum_as_a_service_api_client.api.sessions.delete_session import (
    sync_detailed as _delete_session_sync,
)
from quantum_as_a_service_api_client.api.platforms.list_platforms import (
    sync as _list_platforms_sync,
)
from quantum_as_a_service_api_client.api.platforms.get_platform import (
    sync as _get_platform_sync,
)
from quantum_as_a_service_api_client.api.jobs.create_job import (
    sync as _create_job_sync,
)
from quantum_as_a_service_api_client.api.jobs.get_job import (
    sync as _get_job_sync,
)
from quantum_as_a_service_api_client.api.jobs.list_job_results import (
    sync as _list_job_result_sync,
)

from quantum_as_a_service_api_client.client import Client


__DEFAULT_URL = "https://api.scaleway.com/qaas/v1alpha1"


class QaaSClient:
    def __init__(self, project_id: str, secret_key: str, url: str = __DEFAULT_URL):
        self.__project_id = project_id

        self.__client = Client(
            headers={"X-Auth-Token": secret_key},
            base_url=url,
            timeout=10.0,
            verify_ssl="https" in url,
        )

    def get_platform(self, platform_id: str) -> ScalewayQaasV1Alpha1Platform:
        platform = _get_platform_sync(client=self.__client, platform_id=platform_id)

        return platform

    def list_platforms(
        self, name: Optional[str] = None
    ) -> List[ScalewayQaasV1Alpha1Platform]:
        response = _list_platforms_sync(client=self.__client, name=name)

        assert response

        return response.platforms

    def create_session(
        self,
        platform_id: str,
        max_duration: str,
        max_idle_duration: str,
        deduplication_id: Optional[str] = None,
        name: Optional[str] = None,
    ) -> ScalewayQaasV1Alpha1Session:
        name = name if name else f"qs-{randomname.get_name()}"

        session = _create_session_sync(
            client=self.__client,
            body=CreateSessionBody(
                project_id=self.__project_id,
                name=name,
                platform_id=platform_id,
                deduplication_id=deduplication_id,
                max_duration=max_duration,
                max_idle_duration=max_idle_duration,
            ),
        )

        return session

    def get_session(self, session_id: str) -> ScalewayQaasV1Alpha1Session:
        session = _get_session_sync(client=self.__client, session_id=session_id)

        return session

    def terminate_session(self, session_id: str) -> ScalewayQaasV1Alpha1Session:
        session = _terminate_session_sync(
            client=self.__client,
            body=TerminateSessionBody(
                session_id=session_id,
            ),
        )

        return session

    def delete_session(self, session_id: str):
        _delete_session_sync(client=self.__client, session_id=session_id)

    def create_job(
        self,
        session_id: str,
        payload: Union[Dict, List, str],
        name: Optional[str] = None,
    ) -> ScalewayQaasV1Alpha1Job:
        payload = payload if isinstance(payload, str) else json.dumps(payload)
        name = name if name else f"qj-{randomname.get_name()}"

        job = _create_job_sync(
            client=self.__client,
            body=CreateJobBody(
                name=name,
                session_id=session_id,
                circuit=CreateJobBodyCircuit(qiskit_circuit=payload),
            ),
        )

        return job

    def get_job(self, job_id: str) -> ScalewayQaasV1Alpha1Job:
        job = _get_job_sync(client=self.__client, job_id=job_id)

        return job

    def list_job_results(self, job_id: str) -> List[ScalewayQaasV1Alpha1JobResult]:
        response = _list_job_result_sync(client=self.__client, job_id=job_id)

        return response.job_results



from enum import Enum


class ScalewayQaasV1Alpha1ListSessionACLsRequestOrderBy(str, Enum):
    ACCESS_ASC = "access_asc"
    ACCESS_DESC = "access_desc"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ScalewayQaasV1Alpha1PlatformPricePerHour")


@_attrs_define
class ScalewayQaasV1Alpha1PlatformPricePerHour:
    """Price to be paid per hour (excluding free tiers).

    Attributes:
        currency_code (Union[Unset, str]):
        units (Union[Unset, int]):
        nanos (Union[Unset, int]):
    """

    currency_code: Union[Unset, str] = UNSET
    units: Union[Unset, int] = UNSET
    nanos: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        currency_code = self.currency_code

        units = self.units

        nanos = self.nanos

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if currency_code is not UNSET:
            field_dict["currency_code"] = currency_code
        if units is not UNSET:
            field_dict["units"] = units
        if nanos is not UNSET:
            field_dict["nanos"] = nanos

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        currency_code = d.pop("currency_code", UNSET)

        units = d.pop("units", UNSET)

        nanos = d.pop("nanos", UNSET)

        scaleway_qaas_v1_alpha_1_platform_price_per_hour = cls(
            currency_code=currency_code,
            units=units,
            nanos=nanos,
        )

        scaleway_qaas_v1_alpha_1_platform_price_per_hour.additional_properties = d
        return scaleway_qaas_v1_alpha_1_platform_price_per_hour

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_job_body_circuit import CreateJobBodyCircuit


T = TypeVar("T", bound="CreateJobBody")


@_attrs_define
class CreateJobBody:
    """
    Attributes:
        name (str): Name of the job.
        session_id (str): Session in which the job is executed.
        circuit (CreateJobBodyCircuit): Quantum circuit that should be executed.
        tags (Union[None, Unset, list[str]]): Tags of the job.
        max_duration (Union[None, Unset, str]): Maximum duration of the job. (in seconds) Example: 2.5s.
    """

    name: str
    session_id: str
    circuit: "CreateJobBodyCircuit"
    tags: Union[None, Unset, list[str]] = UNSET
    max_duration: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        session_id = self.session_id

        circuit = self.circuit.to_dict()

        tags: Union[None, Unset, list[str]]
        if isinstance(self.tags, Unset):
            tags = UNSET
        elif isinstance(self.tags, list):
            tags = self.tags

        else:
            tags = self.tags

        max_duration: Union[None, Unset, str]
        if isinstance(self.max_duration, Unset):
            max_duration = UNSET
        else:
            max_duration = self.max_duration

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "session_id": session_id,
                "circuit": circuit,
            }
        )
        if tags is not UNSET:
            field_dict["tags"] = tags
        if max_duration is not UNSET:
            field_dict["max_duration"] = max_duration

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.create_job_body_circuit import CreateJobBodyCircuit

        d = dict(src_dict)
        name = d.pop("name")

        session_id = d.pop("session_id")

        circuit = CreateJobBodyCircuit.from_dict(d.pop("circuit"))

        def _parse_tags(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tags_type_0 = cast(list[str], data)

                return tags_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        tags = _parse_tags(d.pop("tags", UNSET))

        def _parse_max_duration(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        max_duration = _parse_max_duration(d.pop("max_duration", UNSET))

        create_job_body = cls(
            name=name,
            session_id=session_id,
            circuit=circuit,
            tags=tags,
            max_duration=max_duration,
        )

        create_job_body.additional_properties = d
        return create_job_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright 2025 Scaleway
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from enum import Enum
from typing import List, Dict

from dataclasses import dataclass
from dataclasses_json import dataclass_json


class QaaSCircuitSerializationFormat(Enum):
    UNKOWN_CIRCUIT_SERIALIZATION = 0
    QASM_V1 = 1
    QASM_V2 = 2
    QASM_V3 = 3


@dataclass_json
@dataclass
class QaaSCircuitData:
    serialization_format: QaaSCircuitSerializationFormat
    circuit_serialization: str


@dataclass_json
@dataclass
class QaaSJobRunData:
    circuits: List[QaaSCircuitData]
    options: Dict


@dataclass_json
@dataclass
class QaaSJobBackendData:
    name: str
    version: str
    options: Dict


@dataclass_json
@dataclass
class QaaSJobClientData:
    user_agent: str


@dataclass_json
@dataclass
class QaaSJobData:
    client: QaaSJobClientData
    backend: QaaSJobBackendData
    run: QaaSJobRunData



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.scaleway_qaas_v1_alpha_1_session import ScalewayQaasV1Alpha1Session


T = TypeVar("T", bound="ScalewayQaasV1Alpha1ListSessionsResponse")


@_attrs_define
class ScalewayQaasV1Alpha1ListSessionsResponse:
    """
    Attributes:
        total_count (Union[Unset, int]): Total number of sessions.
        sessions (Union[Unset, list['ScalewayQaasV1Alpha1Session']]): List of sessions.
    """

    total_count: Union[Unset, int] = UNSET
    sessions: Union[Unset, list["ScalewayQaasV1Alpha1Session"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_count = self.total_count

        sessions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.sessions, Unset):
            sessions = []
            for sessions_item_data in self.sessions:
                sessions_item = sessions_item_data.to_dict()
                sessions.append(sessions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if total_count is not UNSET:
            field_dict["total_count"] = total_count
        if sessions is not UNSET:
            field_dict["sessions"] = sessions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.scaleway_qaas_v1_alpha_1_session import (
            ScalewayQaasV1Alpha1Session,
        )

        d = dict(src_dict)
        total_count = d.pop("total_count", UNSET)

        sessions = []
        _sessions = d.pop("sessions", UNSET)
        for sessions_item_data in _sessions or []:
            sessions_item = ScalewayQaasV1Alpha1Session.from_dict(sessions_item_data)

            sessions.append(sessions_item)

        scaleway_qaas_v1_alpha_1_list_sessions_response = cls(
            total_count=total_count,
            sessions=sessions,
        )

        scaleway_qaas_v1_alpha_1_list_sessions_response.additional_properties = d
        return scaleway_qaas_v1_alpha_1_list_sessions_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.scaleway_qaas_v1_alpha_1_application_type import (
    ScalewayQaasV1Alpha1ApplicationType,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="ScalewayQaasV1Alpha1Application")


@_attrs_define
class ScalewayQaasV1Alpha1Application:
    """
    Attributes:
        id (Union[Unset, str]): Unique ID of the application.
        name (Union[Unset, str]): Name of the application.
        type_ (Union[Unset, ScalewayQaasV1Alpha1ApplicationType]): Type of the application. Default:
            ScalewayQaasV1Alpha1ApplicationType.UNKNOWN_TYPE.
        compatible_platform_ids (Union[Unset, list[str]]): List of compatible platform (by IDs) able to run this
            application.
        input_template (Union[Unset, str]): JSON format describing the expected input.
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    type_: Union[Unset, ScalewayQaasV1Alpha1ApplicationType] = (
        ScalewayQaasV1Alpha1ApplicationType.UNKNOWN_TYPE
    )
    compatible_platform_ids: Union[Unset, list[str]] = UNSET
    input_template: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        compatible_platform_ids: Union[Unset, list[str]] = UNSET
        if not isinstance(self.compatible_platform_ids, Unset):
            compatible_platform_ids = self.compatible_platform_ids

        input_template = self.input_template

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if type_ is not UNSET:
            field_dict["type"] = type_
        if compatible_platform_ids is not UNSET:
            field_dict["compatible_platform_ids"] = compatible_platform_ids
        if input_template is not UNSET:
            field_dict["input_template"] = input_template

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, ScalewayQaasV1Alpha1ApplicationType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = ScalewayQaasV1Alpha1ApplicationType(_type_)

        compatible_platform_ids = cast(
            list[str], d.pop("compatible_platform_ids", UNSET)
        )

        input_template = d.pop("input_template", UNSET)

        scaleway_qaas_v1_alpha_1_application = cls(
            id=id,
            name=name,
            type_=type_,
            compatible_platform_ids=compatible_platform_ids,
            input_template=input_template,
        )

        scaleway_qaas_v1_alpha_1_application.additional_properties = d
        return scaleway_qaas_v1_alpha_1_application

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.scaleway_qaas_v1_alpha_1_process import ScalewayQaasV1Alpha1Process


T = TypeVar("T", bound="ScalewayQaasV1Alpha1ListProcessesResponse")


@_attrs_define
class ScalewayQaasV1Alpha1ListProcessesResponse:
    """
    Attributes:
        total_count (Union[Unset, int]): Total number of processes.
        processes (Union[Unset, list['ScalewayQaasV1Alpha1Process']]): List of processes.
    """

    total_count: Union[Unset, int] = UNSET
    processes: Union[Unset, list["ScalewayQaasV1Alpha1Process"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_count = self.total_count

        processes: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.processes, Unset):
            processes = []
            for processes_item_data in self.processes:
                processes_item = processes_item_data.to_dict()
                processes.append(processes_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if total_count is not UNSET:
            field_dict["total_count"] = total_count
        if processes is not UNSET:
            field_dict["processes"] = processes

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.scaleway_qaas_v1_alpha_1_process import (
            ScalewayQaasV1Alpha1Process,
        )

        d = dict(src_dict)
        total_count = d.pop("total_count", UNSET)

        processes = []
        _processes = d.pop("processes", UNSET)
        for processes_item_data in _processes or []:
            processes_item = ScalewayQaasV1Alpha1Process.from_dict(processes_item_data)

            processes.append(processes_item)

        scaleway_qaas_v1_alpha_1_list_processes_response = cls(
            total_count=total_count,
            processes=processes,
        )

        scaleway_qaas_v1_alpha_1_list_processes_response.additional_properties = d
        return scaleway_qaas_v1_alpha_1_list_processes_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ScalewayQaasV1Alpha1SessionAccess(str, Enum):
    FULL = "full"
    READ_JOB = "read_job"
    READ_JOB_CIRCUIT = "read_job_circuit"
    READ_JOB_RESULT = "read_job_result"
    READ_SESSION = "read_session"
    READ_WRITE_JOB = "read_write_job"
    READ_WRITE_SESSION = "read_write_session"
    UNKNOWN_ACCESS = "unknown_access"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.list_applications_application_type import ListApplicationsApplicationType
from ...models.list_applications_order_by import ListApplicationsOrderBy
from ...models.scaleway_qaas_v1_alpha_1_list_applications_response import (
    ScalewayQaasV1Alpha1ListApplicationsResponse,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    name: Union[Unset, str] = UNSET,
    application_type: Union[
        Unset, ListApplicationsApplicationType
    ] = ListApplicationsApplicationType.UNKNOWN_TYPE,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListApplicationsOrderBy] = ListApplicationsOrderBy.NAME_ASC,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["name"] = name

    json_application_type: Union[Unset, str] = UNSET
    if not isinstance(application_type, Unset):
        json_application_type = application_type.value

    params["application_type"] = json_application_type

    params["page"] = page

    params["page_size"] = page_size

    json_order_by: Union[Unset, str] = UNSET
    if not isinstance(order_by, Unset):
        json_order_by = order_by.value

    params["order_by"] = json_order_by

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/qaas/v1alpha1/applications",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[ScalewayQaasV1Alpha1ListApplicationsResponse]:
    if response.status_code == 200:
        response_200 = ScalewayQaasV1Alpha1ListApplicationsResponse.from_dict(
            response.json()
        )

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[ScalewayQaasV1Alpha1ListApplicationsResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    name: Union[Unset, str] = UNSET,
    application_type: Union[
        Unset, ListApplicationsApplicationType
    ] = ListApplicationsApplicationType.UNKNOWN_TYPE,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListApplicationsOrderBy] = ListApplicationsOrderBy.NAME_ASC,
) -> Response[ScalewayQaasV1Alpha1ListApplicationsResponse]:
    """List all available applications

     Retrieve information about all applications.

    Args:
        name (Union[Unset, str]): List applications with this name.
        application_type (Union[Unset, ListApplicationsApplicationType]): List applications with
            this type. Default: ListApplicationsApplicationType.UNKNOWN_TYPE.
        page (Union[Unset, int]): Page number.
        page_size (Union[Unset, int]): Maximum number of applications a to return per page.
        order_by (Union[Unset, ListApplicationsOrderBy]): Sort order of the returned applications.
            Default: ListApplicationsOrderBy.NAME_ASC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ScalewayQaasV1Alpha1ListApplicationsResponse]
    """

    kwargs = _get_kwargs(
        name=name,
        application_type=application_type,
        page=page,
        page_size=page_size,
        order_by=order_by,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    name: Union[Unset, str] = UNSET,
    application_type: Union[
        Unset, ListApplicationsApplicationType
    ] = ListApplicationsApplicationType.UNKNOWN_TYPE,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListApplicationsOrderBy] = ListApplicationsOrderBy.NAME_ASC,
) -> Optional[ScalewayQaasV1Alpha1ListApplicationsResponse]:
    """List all available applications

     Retrieve information about all applications.

    Args:
        name (Union[Unset, str]): List applications with this name.
        application_type (Union[Unset, ListApplicationsApplicationType]): List applications with
            this type. Default: ListApplicationsApplicationType.UNKNOWN_TYPE.
        page (Union[Unset, int]): Page number.
        page_size (Union[Unset, int]): Maximum number of applications a to return per page.
        order_by (Union[Unset, ListApplicationsOrderBy]): Sort order of the returned applications.
            Default: ListApplicationsOrderBy.NAME_ASC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ScalewayQaasV1Alpha1ListApplicationsResponse
    """

    return sync_detailed(
        client=client,
        name=name,
        application_type=application_type,
        page=page,
        page_size=page_size,
        order_by=order_by,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    name: Union[Unset, str] = UNSET,
    application_type: Union[
        Unset, ListApplicationsApplicationType
    ] = ListApplicationsApplicationType.UNKNOWN_TYPE,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListApplicationsOrderBy] = ListApplicationsOrderBy.NAME_ASC,
) -> Response[ScalewayQaasV1Alpha1ListApplicationsResponse]:
    """List all available applications

     Retrieve information about all applications.

    Args:
        name (Union[Unset, str]): List applications with this name.
        application_type (Union[Unset, ListApplicationsApplicationType]): List applications with
            this type. Default: ListApplicationsApplicationType.UNKNOWN_TYPE.
        page (Union[Unset, int]): Page number.
        page_size (Union[Unset, int]): Maximum number of applications a to return per page.
        order_by (Union[Unset, ListApplicationsOrderBy]): Sort order of the returned applications.
            Default: ListApplicationsOrderBy.NAME_ASC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ScalewayQaasV1Alpha1ListApplicationsResponse]
    """

    kwargs = _get_kwargs(
        name=name,
        application_type=application_type,
        page=page,
        page_size=page_size,
        order_by=order_by,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    name: Union[Unset, str] = UNSET,
    application_type: Union[
        Unset, ListApplicationsApplicationType
    ] = ListApplicationsApplicationType.UNKNOWN_TYPE,
    page: Union[Unset, int] = UNSET,
    page_size: Union[Unset, int] = UNSET,
    order_by: Union[Unset, ListApplicationsOrderBy] = ListApplicationsOrderBy.NAME_ASC,
) -> Optional[ScalewayQaasV1Alpha1ListApplicationsResponse]:
    """List all available applications

     Retrieve information about all applications.

    Args:
        name (Union[Unset, str]): List applications with this name.
        application_type (Union[Unset, ListApplicationsApplicationType]): List applications with
            this type. Default: ListApplicationsApplicationType.UNKNOWN_TYPE.
        page (Union[Unset, int]): Page number.
        page_size (Union[Unset, int]): Maximum number of applications a to return per page.
        order_by (Union[Unset, ListApplicationsOrderBy]): Sort order of the returned applications.
            Default: ListApplicationsOrderBy.NAME_ASC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ScalewayQaasV1Alpha1ListApplicationsResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            name=name,
            application_type=application_type,
            page=page,
            page_size=page_size,
            order_by=order_by,
        )
    ).parsed



from enum import Enum


class ListApplicationsOrderBy(str, Enum):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    TYPE_ASC = "type_asc"
    TYPE_DESC = "type_desc"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.scaleway_qaas_v1_alpha_1_job_status import ScalewayQaasV1Alpha1JobStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="ScalewayQaasV1Alpha1Job")


@_attrs_define
class ScalewayQaasV1Alpha1Job:
    """
    Attributes:
        id (Union[Unset, str]): Unique ID of the job.
        name (Union[Unset, str]): Job name.
        tags (Union[None, Unset, list[str]]): Tags of the job.
        session_id (Union[Unset, str]): Session ID in which the job is executed.
        created_at (Union[None, Unset, datetime.datetime]): Time at which the job was created. (RFC 3339 format)
            Example: 2022-03-22T12:34:56.123456Z.
        started_at (Union[None, Unset, datetime.datetime]): Time at which the job was started. (RFC 3339 format)
            Example: 2022-03-22T12:34:56.123456Z.
        updated_at (Union[None, Unset, datetime.datetime]): Time at which the job was updated. (RFC 3339 format)
            Example: 2022-03-22T12:34:56.123456Z.
        status (Union[Unset, ScalewayQaasV1Alpha1JobStatus]): Job status. Default:
            ScalewayQaasV1Alpha1JobStatus.UNKNOWN_STATUS.
        progress_message (Union[None, Unset, str]): Last progress message, if the job has started.
        job_duration (Union[None, Unset, str]): Duration of the job, if the job is finished. (in seconds) Example: 2.5s.
        result_distribution (Union[None, Unset, str]): Result of the job, if the job is finished.
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    tags: Union[None, Unset, list[str]] = UNSET
    session_id: Union[Unset, str] = UNSET
    created_at: Union[None, Unset, datetime.datetime] = UNSET
    started_at: Union[None, Unset, datetime.datetime] = UNSET
    updated_at: Union[None, Unset, datetime.datetime] = UNSET
    status: Union[Unset, ScalewayQaasV1Alpha1JobStatus] = (
        ScalewayQaasV1Alpha1JobStatus.UNKNOWN_STATUS
    )
    progress_message: Union[None, Unset, str] = UNSET
    job_duration: Union[None, Unset, str] = UNSET
    result_distribution: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        tags: Union[None, Unset, list[str]]
        if isinstance(self.tags, Unset):
            tags = UNSET
        elif isinstance(self.tags, list):
            tags = self.tags

        else:
            tags = self.tags

        session_id = self.session_id

        created_at: Union[None, Unset, str]
        if isinstance(self.created_at, Unset):
            created_at = UNSET
        elif isinstance(self.created_at, datetime.datetime):
            created_at = self.created_at.isoformat()
        else:
            created_at = self.created_at

        started_at: Union[None, Unset, str]
        if isinstance(self.started_at, Unset):
            started_at = UNSET
        elif isinstance(self.started_at, datetime.datetime):
            started_at = self.started_at.isoformat()
        else:
            started_at = self.started_at

        updated_at: Union[None, Unset, str]
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        progress_message: Union[None, Unset, str]
        if isinstance(self.progress_message, Unset):
            progress_message = UNSET
        else:
            progress_message = self.progress_message

        job_duration: Union[None, Unset, str]
        if isinstance(self.job_duration, Unset):
            job_duration = UNSET
        else:
            job_duration = self.job_duration

        result_distribution: Union[None, Unset, str]
        if isinstance(self.result_distribution, Unset):
            result_distribution = UNSET
        else:
            result_distribution = self.result_distribution

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if tags is not UNSET:
            field_dict["tags"] = tags
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if started_at is not UNSET:
            field_dict["started_at"] = started_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if status is not UNSET:
            field_dict["status"] = status
        if progress_message is not UNSET:
            field_dict["progress_message"] = progress_message
        if job_duration is not UNSET:
            field_dict["job_duration"] = job_duration
        if result_distribution is not UNSET:
            field_dict["result_distribution"] = result_distribution

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_tags(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tags_type_0 = cast(list[str], data)

                return tags_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        tags = _parse_tags(d.pop("tags", UNSET))

        session_id = d.pop("session_id", UNSET)

        def _parse_created_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                created_at_type_0 = isoparse(data)

                return created_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        created_at = _parse_created_at(d.pop("created_at", UNSET))

        def _parse_started_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                started_at_type_0 = isoparse(data)

                return started_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        started_at = _parse_started_at(d.pop("started_at", UNSET))

        def _parse_updated_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)

                return updated_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        updated_at = _parse_updated_at(d.pop("updated_at", UNSET))

        _status = d.pop("status", UNSET)
        status: Union[Unset, ScalewayQaasV1Alpha1JobStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ScalewayQaasV1Alpha1JobStatus(_status)

        def _parse_progress_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        progress_message = _parse_progress_message(d.pop("progress_message", UNSET))

        def _parse_job_duration(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        job_duration = _parse_job_duration(d.pop("job_duration", UNSET))

        def _parse_result_distribution(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        result_distribution = _parse_result_distribution(
            d.pop("result_distribution", UNSET)
        )

        scaleway_qaas_v1_alpha_1_job = cls(
            id=id,
            name=name,
            tags=tags,
            session_id=session_id,
            created_at=created_at,
            started_at=started_at,
            updated_at=updated_at,
            status=status,
            progress_message=progress_message,
            job_duration=job_duration,
            result_distribution=result_distribution,
        )

        scaleway_qaas_v1_alpha_1_job.additional_properties = d
        return scaleway_qaas_v1_alpha_1_job

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
